package spoke.connection;

import java.io.*;
import java.util.*;

public class Connector extends ConnectionSupport {
	protected Connectable connectable;
	protected Hashtable hin;
	protected Hashtable hout;
	protected InputStream in[];
	protected OutputStream out[];
	transient Thread thread;

	public Connector(Connectable connectable, int nin, int nout) {
		super(connectable);
		this.connectable=connectable;
		hin=new Hashtable();
		hout=new Hashtable();
		in=new InputStream[nin];
		out=new OutputStream[nout];
	}

	public InputStream getInputStream(int n) {
		return in[n];
	}

	public OutputStream getOutputStream(int n) {
		return out[n];
	}

	public synchronized void connection(ConnectionEvent ev) {
		Connectable source=(Connectable)ev.getSource();
		String cmd=ev.getCmd();
		int nin=ev.getInput();
		int nout=ev.getOutput();

		if(cmd.equals("start")) {
			if(!hin.containsKey(source)) {
				put(source,nin,nout);
				if(hin.size()==in.length) start();
			}
		} else if(cmd.equals("stop")) {
			if(hin.size()==in.length) stop();
			remove(source);
		}
	}

	public void start() {
		fireConnection("start");
		connectable.open();
		thread=new Thread(connectable);
		thread.start();
	}

	public void stop() {
		connectable.stop();
		thread.interrupt();
		connectable.close();
		fireConnection("stop");
	}

	public void put(Connectable source, int nin, int nout) {
		PipedInputStream is=new PipedInputStream();
		PipedOutputStream os=new PipedOutputStream();

		for(;nin<in.length;nin++) if(in[nin]==null) break;
		hin.put(source,new Integer(nin));
		in[nin]=is;
		try {
			os.connect(is);
		} catch (IOException e) {
			e.printStackTrace();
		}
		source.connect(connectable,os,nout);
	}

	public void remove(Connectable source) {
		source.disconnect(connectable);
		int index=((Integer)hin.get(source)).intValue();
		hin.remove(source);
		in[index]=null;
	}

	public void connect(Connectable dest, OutputStream os, int nout) {
		for(;nout<out.length;nout++) if(out[nout]==null) break;
		hout.put(dest,new Integer(nout));
		out[nout]=os;
	}

	public void disconnect(Connectable dest) {
		int index=((Integer)hout.get(dest)).intValue();
		hout.remove(dest);
		out[index]=null;
	}

	public void setInput(int nin) {
		in=new InputStream[nin];
	}

	public void setOutput(int nout) {
		out=new OutputStream[nout];
	}
}
