package spoke.math.numeric;

public class Matrix extends Arithmetic {
	public double element[][];
	protected int n,p;

	public Matrix(double element[][]) {
		this.element=element;
		n=element.length;
		p=element.length>0?element[0].length:0;
	}

	public Arithmetic add(double d) {
		return add(valueof(d));
	}

	public Arithmetic subtract(double d) {
		return subtract(valueof(d));
	}

	public Arithmetic multiply(double d) {
		Matrix m=(Matrix)newinstance();
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				m.element[i][j]=element[i][j]*d;
			}
		}
		return m;
	}

	public Arithmetic divide(double d) {
		Matrix m=(Matrix)newinstance();
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				m.element[i][j]=element[i][j]/d;
			}
		}
		return m;
	}

	public Arithmetic add(Arithmetic arithmetic) {
		Matrix m=(Matrix)arithmetic;
		Matrix m2=(Matrix)newinstance();
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				m2.element[i][j]=element[i][j]+m.element[i][j];
			}
		}
		return m2;
	}

	public Arithmetic subtract(Arithmetic arithmetic) {
		Matrix m=(Matrix)arithmetic;
		Matrix m2=(Matrix)newinstance();
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				m2.element[i][j]=element[i][j]-m.element[i][j];
			}
		}
		return m2;
	}

	public Arithmetic multiply(Arithmetic arithmetic) {
		if(arithmetic instanceof Matrix) {
			Matrix m=(Matrix)arithmetic;
			Matrix m2=(Matrix)newinstance(n,m.p);
			for(int i=0;i<n;i++) {
				for(int j=0;j<m.p;j++) {
					for(int k=0;k<p;k++) {
						m2.element[i][j]+=element[i][k]*m.element[k][j];
					}
				}
			}
			return m2;
		} else if(arithmetic instanceof SpokeVector) {
			SpokeVector v=(SpokeVector)arithmetic;
			SpokeVector v2=(SpokeVector)((SpokeVector)arithmetic).newinstance(n);
			for(int i=0;i<n;i++) {
				for(int k=0;k<p;k++) {
					v2.element[i]+=element[i][k]*v.element[k];
				}
			}
			return v2;
		} else {
			return null;
		}
	}

	public Arithmetic divide(Arithmetic arithmetic) {
		return null;
	}

	public Arithmetic negate() {
		Matrix m=(Matrix)newinstance();
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				m.element[i][j]=-element[i][j];
			}
		}
		return m;
	}

	public int signum() {
		return 0;
	}

	public Arithmetic valueof(double d) {
		Matrix m=(Matrix)identity(Math.max(n,p)).multiply(d);
		Matrix m2=(Matrix)newinstance();
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				m2.element[i][j]=m.element[i][j];
			}
		}
		return m2;
	}

	public Arithmetic tensorProduct(Matrix matrix) {
		Matrix m=(Matrix)newinstance(n*matrix.n,p*matrix.p);
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				for(int k=0;k<matrix.n;k++) {
					for(int l=0;l<matrix.p;l++) {
						m.element[i*matrix.n+k][j*matrix.p+l]=element[i][j]*matrix.element[k][l];
					}
				}
			}
		}
		return m;
	}

	public Arithmetic transpose() {
		Matrix m=(Matrix)newinstance(p,n);
		for(int i=0;i<n;i++) {
			for(int j=0;j<p;j++) {
				m.element[j][i]=element[i][j];
			}
		}
		return m;
	}

	public double trace() {
		double d=1.;
		for(int i=0;i<n;i++) {
			d*=element[i][i];
		}
		return d;
	}

	public double determinant() {
		if(n>1) {
			double d=0.;
			for(int i=0;i<n;i++) {
				if(element[i][0]==0.);
				else {
					Matrix m=new Matrix(new double[n-1][n-1]);
					for(int j=0;j<n-1;j++) {
						for(int k=0;k<n-1;k++) m.element[j][k]=element[j<i?j:j+1][k+1];
					}
					if(i%2==0) d+=element[i][0]*m.determinant();
					else d-=element[i][0]*m.determinant();
				}
			}
			return d;
		} else if(n>0) return element[0][0];
		else return 0.;
	}

	public static Matrix identity(int dimension) {
		Matrix m=new Matrix(new double[dimension][dimension]);
		for(int i=0;i<m.n;i++) {
			for(int j=0;j<m.p;j++) {
				if(i==j) {
					m.element[i][j]=1.;
				} else {
					m.element[i][j]=0.;
				}
			}
		}
		return m;
	}

	public static Matrix frame(SpokeVector vector[]) {
		Matrix m=new Matrix(new double[vector.length>0?vector[0].n:0][vector.length]);
		for(int i=0;i<m.n;i++) {
			for(int j=0;j<m.p;j++) m.element[i][j]=vector[j].element[i];
		}
		return m;
	}

	public static Matrix rotation(int dimension, int plane, double angle) {
		return rotation(dimension,plane,2,angle);
	}

	public static Matrix rotation(int dimension, int axis1, int axis2, double angle) {
		Matrix m=new Matrix(new double[dimension][dimension]);
		for(int i=0;i<m.n;i++) {
			for(int j=0;j<m.p;j++) {
				if(i==axis1 && j==axis1) {
					m.element[i][j]=Math.cos(angle);
				} else if(i==axis1 && j==axis2) {
					m.element[i][j]=-Math.sin(angle);
				} else if(i==axis2 && j==axis1) {
					m.element[i][j]=Math.sin(angle);
				} else if(i==axis2 && j==axis2) {
					m.element[i][j]=Math.cos(angle);
				} else if(i==j) {
					m.element[i][j]=1.;
				} else {
					m.element[i][j]=0.;
				}
			}
		}
		return m;
	}

	protected Arithmetic newinstance() {
		return newinstance(n,p);
	}

	protected Arithmetic newinstance(int n, int p) {
		return new Matrix(new double[n][p]);
	}
}
