package spoke.math.numeric;

public class SpokeVector extends Arithmetic {
	public double element[];
	protected int n;

	public SpokeVector(double element[]) {
		this.element=element;
		n=element.length;
	}

	public Arithmetic add(double d) {
		return add(valueof(d));
	}

	public Arithmetic subtract(double d) {
		return subtract(valueof(d));
	}

	public Arithmetic multiply(double d) {
		SpokeVector v=(SpokeVector)newinstance();
		for(int i=0;i<n;i++) v.element[i]=element[i]*d;
		return v;
	}

	public Arithmetic divide(double d) {
		SpokeVector v=(SpokeVector)newinstance();
		for(int i=0;i<n;i++) v.element[i]=element[i]/d;
		return v;
	}

	public Arithmetic add(Arithmetic arithmetic) {
		SpokeVector v=(SpokeVector)arithmetic;
		SpokeVector v2=(SpokeVector)newinstance();
		for(int i=0;i<n;i++) v2.element[i]=element[i]+v.element[i];
		return v2;
	}

	public Arithmetic subtract(Arithmetic arithmetic) {
		SpokeVector v=(SpokeVector)arithmetic;
		SpokeVector v2=(SpokeVector)newinstance();
		for(int i=0;i<n;i++) v2.element[i]=element[i]-v.element[i];
		return v2;
	}

	public Arithmetic multiply(Arithmetic arithmetic) {
		if(arithmetic instanceof SpokeVector) {
			return SpokeDouble.valueOf(scalarProduct((SpokeVector)arithmetic));
		} else if(arithmetic instanceof Matrix) {
			return ((Matrix)arithmetic).transpose().multiply(this);
		} else {
			return multiply(((SpokeDouble)arithmetic).content);
		}
	}

	public Arithmetic divide(Arithmetic arithmetic) {
		if(arithmetic instanceof SpokeVector) {
			return null;
		} else if(arithmetic instanceof Matrix) {
			return null;
		} else {
			return divide(((SpokeDouble)arithmetic).content);
		}
	}

	public Arithmetic negate() {
		SpokeVector v=(SpokeVector)newinstance();
		for(int i=0;i<n;i++) v.element[i]=-element[i];
		return v;
	}

	public int signum() {
		return 0;
	}

	public Arithmetic valueof(double d) {
		SpokeVector v=(SpokeVector)unity(n).multiply(d);
		SpokeVector v2=(SpokeVector)newinstance();
		for(int i=0;i<n;i++) v2.element[i]=v.element[i];
		return v2;
	}

	public SpokeVector normalize() {
		return (SpokeVector)multiply(1./norm());
	}

	public double norm() {
		return Math.sqrt(scalarProduct(this));
	}

	public double norm2() {
		return scalarProduct(this);
	}

	public double scalarProduct(SpokeVector vector) {
		double s=0.;
		for(int i=0;i<n;i++) {
			s+=element[i]*vector.element[i];
		}
		return s;
	}

	public SpokeVector vectorProduct(SpokeVector vector) {
		SpokeVector v2=(SpokeVector)newinstance();
		double m[][]={
			{0.,-element[2],element[1]},
			{element[2],0.,-element[0]},
			{-element[1],element[0],0}
		};
		SpokeVector v=(SpokeVector)new Matrix(m).multiply(vector);
		for(int i=0;i<n;i++) v2.element[i]=i<v.n?v.element[i]:0;
		return v2;
	}

	public static SpokeVector unity(int dimension) {
		SpokeVector v=new SpokeVector(new double[dimension]);
		for(int i=0;i<v.n;i++) {
			if(i==0) v.element[i]=1.;
			else v.element[i]=0.;
		}
		return v;
	}

	protected Arithmetic newinstance() {
		return newinstance(n);
	}

	protected Arithmetic newinstance(int n) {
		return new SpokeVector(new double[n]);
	}
}
