package spoke.signal;

import java.io.*;
import spoke.math.numeric.*;

public
class ComplexPipedInputStream extends InputStream implements ComplexInput {
    boolean closed = true;
    boolean closedByReader = false;
    boolean connected = false;

    Thread readSide;
    Thread writeSide;

    protected static final int PIPE_SIZE = 1024;

    protected Complex buffer[] = new Complex[PIPE_SIZE];

    protected int in = -1;

    protected int out = 0;

    public ComplexPipedInputStream(ComplexPipedOutputStream src) throws IOException {
	connect(src);
    }

    public ComplexPipedInputStream() {
    }

    public void connect(ComplexPipedOutputStream src) throws IOException {
	if (connected) {
	    throw new IOException("Pipe already connected");
	}
	src.connect(this);
	connected = true;
    }
    
    protected synchronized void receive(Complex b) throws IOException {
	writeSide = Thread.currentThread();
	while (in == out) {
	    if ((readSide != null) && !readSide.isAlive()) {
		throw new IOException("Pipe broken");
	    }
	    notifyAll();	
	    try {
	        wait(1000);
	    } catch (InterruptedException ex) {
		throw new java.io.InterruptedIOException();
	    }
	}
	if (in < 0) {
	    in = 0;
	    out = 0;
	}
	buffer[in++] = b;
	if (in >= buffer.length) {
	    in = 0;
	}
    }

    synchronized void receive(byte b[], int off, int len)  throws IOException {
/*	while (--len >= 0) {
	    receive(b[off++]);
	}
*/    }

    synchronized void receivedLast() {
	closed = true;
	notifyAll();
    }

    public synchronized int read()  throws IOException {
		return -1;
	}

    public synchronized Complex readComplex()  throws IOException {
	if (closedByReader) {
	    throw new IOException("InputStream closed");
	}
	int trials = 2;
	while (in < 0) {
	    readSide = Thread.currentThread();
	    if (closed) { 
		return null;
	    }
	    if ((writeSide != null) && (!writeSide.isAlive()) && (--trials < 0)) {
		throw new IOException("Pipe broken");
	    }
	    notifyAll();
	    try {
	        wait(1000);
	    } catch (InterruptedException ex) {
		throw new java.io.InterruptedIOException();
	    }
 	}
	Complex ret = buffer[out++];
	if (out >= buffer.length) {
	    out = 0;
	}
	if (in == out) {
	    in = -1;		
	}
	return ret;
    }

    public synchronized int read(byte b[], int off, int len)  throws IOException {
/*
	if (b == null) {
	    throw new NullPointerException();
	} else if (off < 0 || len < 0 || off + len > b.length) {
	    throw new ArrayIndexOutOfBoundsException();
	} else if (len == 0) {
	    return 0;
	}

	int c = read();		
	if (c < 0) {
	    return -1;
	}
	b[off] = (byte) c;
	int rlen = 1;
	while ((in >= 0) && (--len > 0)) {
	    b[off + rlen] = buffer[out++];
	    rlen++;
	    if (out >= buffer.length) {
		out = 0;
	    }
	    if (in == out) {
		in = -1;	
	    }
	}
	return rlen;
*/	return -1;
    }

  public synchronized int available() throws IOException {
    if(in < 0)
      return 0;
    else if(in == out)
      return buffer.length;
    else if (in > out)
      return in - out;
    else
      return in + buffer.length - out;
  }
    
    public void close()  throws IOException {
	in = -1;
	closedByReader = true;
    }
}
