package spoke.signal.chaos;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import spoke.connection.*;
import spoke.signal.*;
import spoke.math.numeric.*;

public class LorenzBean extends Button implements Connectable, ActionListener {
	Connector connector=new ComplexConnector(this,0,1);
	double r=50.0;
	double sigma=10.0;
	double b=8.0/3.0;
	double dt=0.001;
	int coordonnee=0;
	transient boolean state;
	transient LZ lz;
	transient boolean stopped;

	public LorenzBean() {
		setLabel(getClass().getName());
		addActionListener(this);
	}

	public void run() {
		ComplexOutput out=(ComplexOutput)connector.getOutputStream(0);

		stopped=false;
		while(!stopped) {
			try {
				out.writeComplex(Complex.valueOf(lz.compute()[coordonnee],0.));
				lz.next();
			} catch (InterruptedIOException e) {
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void stop() {
		stopped=true;
	}

	public void init() {
		lz=new LZ(r,sigma,b,dt);
	}

	public void open() {
		init();
	}

	public void close() {}

	public void connect(Connectable dest, OutputStream os, int nout) {
		connector.connect(dest,os,nout);
	}

	public void disconnect(Connectable dest) {
		connector.disconnect(dest);
	}

	public void actionPerformed(ActionEvent e) {
		if(state==false) {
			connector.start();
			state=true;
		} else {
			state=false;
			connector.stop();
		}
	}

	public synchronized void addConnectionListener(ConnectionListener l) {
		connector.addConnectionListener(l);
	}

	public synchronized void removeConnectionListener(ConnectionListener l) {
		connector.removeConnectionListener(l);
	}

	public void setR(double r) {
		this.r=r;
		init();
	}

	public double getR() {
		return(r);
	}

	public void setSigma(double sigma) {
		this.sigma=sigma;
		init();
	}

	public double getSigma() {
		return(sigma);
	}

	public void setB(double b) {
		this.b=b;
		init();
	}

	public double getB() {
		return(b);
	}

	public void setDt(double dt) {
		this.dt=dt;
		init();
	}

	public double getDt() {
		return(dt);
	}

	public void setCoordonnee(int coordonnee) {
		this.coordonnee=coordonnee;
		this.coordonnee=coordonnee<0?0:(coordonnee>2?2:coordonnee);
	}

	public int getCoordonnee() {
		return(coordonnee);
	}
}
