package spoke.signal.kohonen;

import java.io.*;
import java.util.*;
import spoke.util.*;

public class LVQ implements Serializable {
	public int dimension;
	public Node node[];
	protected double l;
	protected int[][] classes;
	transient Betatree beta;
	transient int nombre;

	public LVQ(int dimension, int nombre, String codes[]) {
		this.dimension=dimension;
		node=new Node[nombre];
		for(int i=0;i<node.length;i++) node[i]=new Node(new double[dimension],"");
		beta=new Betatree(StringComparator.comparator);
		for(int i=0;i<codes.length;i++) {
			if(!beta.containsKey(codes[i])) {
				beta.put(codes[i],new Classe());
			}
			Classe c=(Classe)beta.get(codes[i]);
			c.nb++;
		}
		classes=new int[beta.size()][];
		Enumeration e=beta.elements();
		int n=0;
		for(int i=0;i<classes.length;i++) {
			Classe c=(Classe)e.nextElement();
			classes[i]=new int[i<classes.length-1?(c.nb*nombre)/codes.length:nombre-n];
			c.node=classes[i];
			n+=c.node.length;
		}
		l=.2;
	}

	public void build(double sample[], String code) {
		Classe c=(Classe)beta.get(code);
		if(c.nombre<c.node.length) {
			double weight[]=new double[dimension];
			for(int i=0;i<dimension;i++) weight[i]=sample[i];
			node[nombre]=new Node(weight,code);
			c.node[c.nombre]=nombre;
			c.nombre++;
			nombre++;
		}
	}

	public void learn(double sample[], String code, double lability) {
		int w=use(sample);
		double l=this.l*lability;

		if(node[w].code.compareTo(code)==0) {
			node[w].move(sample,l,false);
		} else {
			node[w].move(sample,l,true);
		}
	}

	public int use(double sample[]) {
		int w=0;
		double dw=0.;

		for(int i=0;i<node.length;i++) {
			double d=node[i].distance(sample);
			if(i>0?d<dw:true) {
				w=i;
				dw=d;
			}
		}
		return w;
	}

	public double[][][] segment() {
		Vector v=new Vector();

		for(int i=0;i<classes.length;i++) {
			int c[]=classes[i];
			for(int j=0;j<c.length;j++) {
				double d[][]=new double[2][dimension];
				for(int k=0;k<dimension;k++) {
					d[0][k]=node[c[j]].weight[k];
					d[1][k]=node[c[(j+1)%c.length]].weight[k];
				}
				v.addElement(d);
			}
		}
		double d[][][]=new double[v.size()][][];
		v.copyInto(d);
		return d;
	}
}

class Classe {
	int nb;
	int node[];
	int nombre;
}
