package spoke.signal.kohonen;

import java.io.*;
import java.util.*;
import spoke.signal.*;
import spoke.math.numeric.*;

public class LearningVectorQuantization extends Module {
	int dimension=0;
	double minimum=-1.0;
	double maximum=1.0;
	int nombre=0;
	boolean build=true;
	boolean learn=true;
	boolean diabolo=true;
	String name="";
	double seuil=0.1;
	public LVQ net=new LVQ(dimension,nombre,new String[0]);
	transient double sample[];
	transient String codes[];

	public void run() {
		ComplexInput in=(ComplexInput)connector.getInputStream(0);
		ComplexOutput out=(ComplexOutput)connector.getOutputStream(0);

		stopped=false;
		for(int k=0;(learn?k<codes.length:true) && !stopped;k++) {
			try {
				for(int i=0;i<sample.length;i++) sample[i]=sample(in.readComplex());
				if(learn) {
					if(build) {
						net.build(sample,codes[k]);
					} else {
						net.learn(sample,codes[k],(double)(codes.length-k)/(double)codes.length);
					}
				} else {
					int n=net.use(sample);
					Node node=net.node[n];
					if(node.distance(sample)<seuil) detection(node.code);
					if(diabolo) {
						double s[]=node.weight;
						for(int i=0;i<s.length;i++) out.writeComplex(reverse(s[i]));
					} else {
						for(int i=0;i<nombre;i++) out.writeComplex(Complex.valueOf(i==n?1.:0.,0.));
					}
				}
			} catch (InterruptedIOException e) {
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		try {
			synchronized(this) {
				while(!stopped) wait();
			}
		} catch (InterruptedException e) {}
	}

	public void detection(String code) {
		System.out.println(code);
	}

	double sample(Complex c) {
		double d=c.element0;
		if(d>maximum) d=1.0;
		else if(d<minimum) d=0.0;
		else d=(d-minimum)/(maximum-minimum);
		return d*2.-1.;
	}

	Complex reverse(double d) {
		return Complex.valueOf(((d+1.)*.5)*(maximum-minimum)+minimum,0.);
	}

	public void init() {
		sample=new double[dimension];
		codes=learn?codes():new String[0];
		net=new LVQ(dimension,nombre,codes);
	}

	public void open() {
		sample=new double[dimension];
		codes=learn?codes():new String[0];
	}

	String[] codes() {
		Vector v=new Vector();
		try {
			InputStream is=new FileInputStream(name);
			Reader r=new BufferedReader(new InputStreamReader(is));
			StreamTokenizer st=new StreamTokenizer(r);
			st.ordinaryChars('0','9');
			st.ordinaryChar('.');
			st.ordinaryChar('-');
			boucle: while(true) {
				switch(st.nextToken()) {
				case StreamTokenizer.TT_WORD:
					v.addElement(st.sval);
					break;
				case StreamTokenizer.TT_EOF:
					break boucle;
				}
			}
		} catch (IOException ex) {
//			ex.printStackTrace();
		}
		String s[]=new String[v.size()];
		v.copyInto(s);
		return s;
	}

	public void setDimension(int dimension) {
		this.dimension=dimension;
		init();
	}

	public int getDimension() {
		return(dimension);
	}

	public void setMinimum(double minimum) {
		this.minimum=minimum;
	}

	public double getMinimum() {
		return(minimum);
	}

	public void setMaximum(double maximum) {
		this.maximum=maximum;
	}

	public double getMaximum() {
		return(maximum);
	}

	public void setNombre(int nombre) {
		this.nombre=nombre;
		init();
	}

	public int getNombre() {
		return(nombre);
	}

	public void setBuild(boolean build) {
		this.build=build;
	}

	public boolean getBuild() {
		return build;
	}

	public void setLearn(boolean learn) {
		this.learn=learn;
	}

	public boolean getLearn() {
		return learn;
	}

	public void setDiabolo(boolean diabolo) {
		this.diabolo=diabolo;
	}

	public boolean getDiabolo() {
		return diabolo;
	}

	public void setCodes(String name) {
		this.name=name;
		init();
	}

	public String getCodes() {
		return name;
	}

	public void setSeuil(double seuil) {
		this.seuil=seuil;
	}

	public double getSeuil() {
		return(seuil);
	}
}
