package spoke.signal.kohonen;

import java.io.*;
import java.util.*;
import spoke.util.*;

public abstract class SOM implements Serializable {
	public int dimension;
	public SOMNode node[];
	protected double l;
	protected double r;

	class Voisinage extends Betatree {
		Vector v;

		Voisinage(int n, double r) {
			v=new Vector();
			put(new Proximite(n));
			for(int i=0;i<(int)r;i++) {
				Vector v0=v;
				v=new Vector();
				put(v0);
			}
		}

		void put(Vector v) {
			for(Enumeration e=v.elements();e.hasMoreElements();) {
				Relation q=(Relation)e.nextElement();
				for(Enumeration f=node[q.node].elements();f.hasMoreElements();) put(q.compose((Relation)f.nextElement()));
			}
		}

		void put(Relation q) {
			if(containsKey(q));
			else {
				put(q,null);
				v.addElement(q);
			}
		}
	}

	public SOM(int dimension) {
		this.dimension=dimension;
	}

	public void learn(double sample[], String code, double lability) {
		int w=use(sample);
		double r=this.r*lability;
		double l=this.l*lability;
		Betatree voisinage=new Voisinage(w,r);

		node[w].code=code;
		for(Enumeration e=voisinage.keys();e.hasMoreElements();) {
			Relation q=(Relation)e.nextElement();
			node[q.node].move(q.fn(sample),l);
		}
	}

	public int use(double sample[]) {
		int w=0;
		double dw=0.;

		for(int i=0;i<node.length;i++) {
			double d=node[i].distance(sample);
			if(i>0?d<dw:true) {
				w=i;
				dw=d;
			}
		}
		return w;
	}

	public double[][][] segment() {
		Vector v=new Vector();

		for(int i=0;i<node.length;i++) {
			for(Enumeration e=node[i].elements();e.hasMoreElements();) {
				Relation q=(Relation)e.nextElement();
				if(q.node<i && q instanceof Proximite) {
					double d[][]=new double[2][dimension];
					for(int k=0;k<dimension;k++) {
						d[0][k]=node[i].weight[k];
						d[1][k]=node[q.node].weight[k];
					}
					v.addElement(d);
				}
			}
		}
		double d[][][]=new double[v.size()][][];
		v.copyInto(d);
		return d;
	}

	public double[][][] surface() {
		return new double[0][][];
	}
}
