package spoke.signal.ondelette;

import spoke.math.numeric.*;

public class ATR {
	static final double ALPHA=.707106781187;
	static final double PREINTEGRATEUR_F[]={.5,.5};
	static final int G_LENGTH=32;
	int n2,n;
	int nu2,nu;
	int voie;
	spoke.signal.Filtre g[];
	spoke.signal.Filtre preintegrateur;
	TOR tor;

	public ATR(int n2, int resume, int voie) {
		this.n2=n2;
		nu2=log2e(n2);
		nu=nu2-resume;
		n=1<<nu;
		this.voie=voie;
		g=new spoke.signal.Filtre[voie];
		for(int k=0;k<voie;k++) {
			double a=Math.pow(2.,(double)k/(double)voie);
			double f[]=new double[G_LENGTH];
			for(int i=0;i<f.length;i++) {
				double t=2.*Math.PI*(((double)i+.5)/(double)f.length)-Math.PI;
				f[i]=morlet(t/a)/Math.sqrt(a);
			}
			g[k]=new spoke.signal.Filtre(f);
		}
		preintegrateur=new spoke.signal.Filtre(PREINTEGRATEUR_F);
		tor=new TOR(n2,new Daubechies(1));
	}

	public void compute(Arithmetic in[], Arithmetic out[][], int octave) {
		Arithmetic x[]=new Complex[n2];
		Arithmetic y[]=new Complex[n2];

		for(int k=0;k<voie;k++) {
			for(int j=0;j<n2;j++) x[j]=in[j];
			octave1(x,y,k);
			for(int i=0;i<octave;i++) {
				tor.compute(y,nu2-nu,false);
				for(int j=0,d=1<<(nu2-nu);j<n;j++) out[(octave-i)*voie-1-k][j]=y[j].divide((double)d);
				octave(x,y,i,k);
			}
		}
	}

	protected void octave1(Arithmetic in[], Arithmetic out[],int voie) {
		for(int j=0;j<n2;j++) out[j]=in[j];
		g[voie].convolution(out);
	}

	protected void octave(Arithmetic in[], Arithmetic out[], int octave, int voie) {
		int nso=n2>>octave;
		int nuo=1<<octave;
		Arithmetic x[]=new Complex[nso];
		Arithmetic y[]=new Complex[nso];

		for(int i=0;i<nuo;i++) {
			int k=i*nso;
			for(int j=0;j<nso;j++) x[j]=in[k+j];
			cellule_elementaire(x,y,voie);
			for(int j=0;j<nso;j++) in[k+j]=x[j];
			k=ret(i,octave);
			for(int j=0;j<nso;j++) out[j*nuo+k]=y[j];
		}
	}

	protected void cellule_elementaire(Arithmetic in[], Arithmetic out[], int voie) {
		int nso=in.length;
		int ns2=nso>>1;
		Arithmetic x[]=new Complex[nso];
		Arithmetic y[]=new Complex[ns2];

		for(int i=0;i<ns2;i++) x[i]=in[i<<1].multiply(ALPHA);
		for(int i=0;i<ns2;i++) x[i+ns2]=in[(i<<1)+1].multiply(ALPHA);
		for(int i=0;i<ns2;i++) y[i]=x[i];
		preintegrateur.convolution(y);
		for(int i=0;i<ns2;i++) in[i]=y[(i+1)%ns2].add(x[i+ns2]);
		for(int i=0;i<ns2;i++) y[i]=x[i+ns2];
		preintegrateur.convolution(y);
		for(int i=0;i<ns2;i++) in[i+ns2]=y[(i+1)%ns2].add(x[(i+1)%ns2]);
		for(int i=0;i<ns2;i++) y[i]=in[i];
		g[voie].convolution(y);
		for(int i=0;i<ns2;i++) out[i<<1]=y[i];
		for(int i=0;i<ns2;i++) y[i]=in[i+ns2];
		g[voie].convolution(y);
		for(int i=0;i<ns2;i++) out[(i<<1)+1]=y[i];
	}

	protected static double morlet(double t) {
		return Math.sin(5.*t)*Math.exp(.5*-t*t);
	}

	protected static int ret(int n, int nu) {
		int p=0;
		for(int i=0;i<nu;i++) {
			p<<=1;
			p+=(n&1);
			n>>=1;
		}
		return p;
	}

	protected static int log2e(int n) {
		int i;
		for(i=0;n>1;n>>=1) i++;
		return i;
	}
}
