package spoke.signal.util;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import spoke.connection.*;
import spoke.signal.*;
import spoke.math.numeric.*;

public class Diagramme extends Panel implements Connectable {
	Connector connector=new ComplexConnector(this,1,0);
	Canvas canvas;
	int largeur=1;
	int hauteur=1;
	double minimum=-1.0;
	double maximum=1.0;
	boolean imaginaire=false;
	boolean couleur=false;
	transient Complex buffer[];
	transient int pixels[];
	transient MemoryImageSource source;
	transient Image image;
	transient int posx;
	transient boolean stopped;

	class Canvas extends java.awt.Canvas {
		public Canvas() {
			setBackground(Color.black);
		}

		public void paint(Graphics g) {
			Dimension s=getSize();
			g.drawImage(image,0,0,s.width,s.height,posx,0,posx+largeur,hauteur,this);
//			synchronized(this) {
//				notify();
//			}
		}

		public void update(Graphics g) {
			paint(g);
		}

		public Dimension getPreferredSize() {
			return getParent().getSize();
		}
	};

	public Diagramme() {
		setSize(128,96);
		setLayout(new BorderLayout());
		canvas=new Canvas();
		add(canvas);
		init();
	}

	public void run() {
		ComplexInput in=(ComplexInput)connector.getInputStream(0);

		stopped=false;
		while(!stopped) {
			try {
				for(int x=0;x<largeur;x++) {
					for(int y=0;y<hauteur;y++) {
						Complex c=in.readComplex();
						int i=largeur*(hauteur-1-y)+x;
						buffer[i]=c;
						i=(largeur<<1)*(hauteur-1-y)+x;
						pixels[i+largeur]=pixels(c);
						pixels[i]=pixels(c);
					}
//					synchronized(canvas) {
						posx=x+1;
						source.newPixels(x+largeur,0,x+1+largeur,hauteur);
						source.newPixels(x,0,x+1,hauteur);
//						try {
//							canvas.wait();
//						} catch (InterruptedException ex) {
//							throw new java.io.InterruptedIOException();
//						}
//					}
				}
			} catch (InterruptedIOException e) {
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void stop() {
		stopped=true;
	}

	int pixels(Complex c) {
		double d=imaginaire?c.element1:c.element0;
		if(d>maximum) d=1.0;
		else if(d<minimum) d=0.0;
		else d=(d-minimum)/(maximum-minimum);
		return couleur?(d==0.0?0xff000000:Color.HSBtoRGB((float)d,1,1)):Color.HSBtoRGB(0,0,(float)d);
	}

	void newBuffer() {
		for(int x=0;x<largeur;x++) {
			for(int y=0;y<hauteur;y++) {
				int i=largeur*(hauteur-1-y)+x;
				Complex c=buffer[i];
				i=(largeur<<1)*(hauteur-1-y)+x;
				pixels[i+largeur]=pixels(c);
				pixels[i]=pixels(c);
			}
		}
	}

	public void init() {
		buffer=new Complex[largeur*hauteur];
		pixels=new int[(largeur<<1)*hauteur];
		for(int i=0;i<buffer.length;i++) buffer[i]=Complex.valueOf(0.,0.);
		newBuffer();
		source=new MemoryImageSource(largeur<<1,hauteur,pixels,0,largeur<<1);
		source.setAnimated(true);
		image=createImage(source);
	}

	public void open() {
		for(int i=0;i<buffer.length;i++) buffer[i]=Complex.valueOf(0.,0.);
		newBuffer();
	}

	public void close() {}

	public void connect(Connectable dest, OutputStream os, int nout) {
		connector.connect(dest,os,nout);
	}

	public void disconnect(Connectable dest) {
		connector.disconnect(dest);
	}

	public void connection(ConnectionEvent e) {
		connector.connection(e);
	}

	public void setLargeur(int largeur) {
		this.largeur=largeur;
		doLayout();
		init();
	}

	public int getLargeur() {
		return(largeur);
	}

	public void setHauteur(int hauteur) {
		this.hauteur=hauteur;
		doLayout();
		init();
	}

	public int getHauteur() {
		return(hauteur);
	}

	public void setMinimum(double minimum) {
		this.minimum=minimum;
		newBuffer();
		source.newPixels();
	}

	public double getMinimum() {
		return(minimum);
	}

	public void setMaximum(double maximum) {
		this.maximum=maximum;
		newBuffer();
		source.newPixels();
	}

	public double getMaximum() {
		return(maximum);
	}

	public void setImaginaire(boolean imaginaire) {
		this.imaginaire=imaginaire;
		newBuffer();
		source.newPixels();
	}

	public boolean getImaginaire() {
		return(imaginaire);
	}

	public void setCouleur(boolean couleur) {
		this.couleur=couleur;
		newBuffer();
		source.newPixels();
	}

	public boolean getCouleur() {
		return(couleur);
	}

	private void writeObject(ObjectOutputStream s) throws IOException {
		s.defaultWriteObject();
	}

	private void readObject(ObjectInputStream s) throws ClassNotFoundException, IOException {
		s.defaultReadObject();
		init();
	}
}
