package spoke.signal.util;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import spoke.connection.*;
import spoke.signal.*;
import spoke.math.numeric.*;

public class Generator extends Button implements Connectable, ActionListener {
	static final Timbre TIMBRE[]={new Sinus(),new Scie(),new Triangle(),new Carre()};
	Connector connector=new ComplexConnector(this,0,1);
	int timbre=0;
	long longueur=0;
	double frequence=0.0;
	double phase=0.0;
	double amplitude=1.0;
	double offset=0.0;
	transient boolean state;
	transient boolean stopped;

	public Generator() {
		setLabel(getClass().getName());
		addActionListener(this);
	}

	public void run() {
		ComplexOutput out=(ComplexOutput)connector.getOutputStream(0);

		stopped=false;
		for(long n=0;(longueur>0?n<longueur:true) && !stopped;n++) {
			try {
				out.writeComplex(Complex.valueOf(TIMBRE[timbre].fn(2.*Math.PI*frequence*(double)n+phase)*amplitude+offset,0.));
			} catch (InterruptedIOException e) {
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		try {
			synchronized(this) {
				while(!stopped) wait();
			}
		} catch (InterruptedException e) {}
	}

	public void stop() {
		stopped=true;
	}

	public void open() {}

	public void close() {}

	public void connect(Connectable dest, OutputStream os, int nout) {
		connector.connect(dest,os,nout);
	}

	public void disconnect(Connectable dest) {
		connector.disconnect(dest);
	}

	public void actionPerformed(ActionEvent e) {
		if(state==false) {
			connector.start();
			state=true;
		} else {
			state=false;
			connector.stop();
		}
	}

	public synchronized void addConnectionListener(ConnectionListener l) {
		connector.addConnectionListener(l);
	}

	public synchronized void removeConnectionListener(ConnectionListener l) {
		connector.removeConnectionListener(l);
	}

	public void setTimbre(int timbre) {
		this.timbre=timbre<0?0:(timbre>=TIMBRE.length?TIMBRE.length-1:timbre);
	}

	public int getTimbre() {
		return timbre;
	}

	public void setLongueur(long longueur) {
		this.longueur=longueur;
	}

	public long getLongueur() {
		return(longueur);
	}

	public void setFrequence(double frequence) {
		this.frequence=frequence;
	}

	public double getFrequence() {
		return frequence;
	}

	public void setPhase(double phase) {
		this.phase=phase;
	}

	public double getPhase() {
		return phase;
	}

	public void setAmplitude(double amplitude) {
		this.amplitude=amplitude;
	}

	public double getAmplitude() {
		return amplitude;
	}

	public void setOffset(double offset) {
		this.offset=offset;
	}

	public double getOffset() {
		return offset;
	}
}

abstract class Timbre {
	abstract double fn(double x);

	protected static double mod2pi(double x) {
		return x-2.*Math.PI*Math.rint(x/(2.*Math.PI));
	}
}

class Sinus extends Timbre {
	double fn(double x) {
		return Math.sin(x);
	}
}

class Scie extends Timbre {
	double fn(double x) {
		return mod2pi(x)/Math.PI;
	}
}

class Triangle extends Timbre {
	double fn(double x) {
		double y=mod2pi(x)/Math.PI;
		if(Math.abs(y)<.5) {
			return y*2.;
		} else {
			if(y<0.) return -(y+1.)*2.;
			else return (1.-y)*2.;
		}
	}
}

class Carre extends Timbre {
	double fn(double x) {
		if(mod2pi(x)<0.) return -1.;
		else return 1.;
	}
}
