package spoke.signal.util;

import java.io.*;
import java.awt.*;
import spoke.connection.*;
import spoke.signal.*;
import spoke.math.numeric.*;

public class Oscilloscope extends Panel implements Connectable {
	Connector connector=new ComplexConnector(this,1,0);
	Canvas canvas;
	int largeur=1;
	double minimum=-1.0;
	double maximum=1.0;
	boolean imaginaire=false;
	transient Complex buffer[];
	transient boolean stopped;

	class Canvas extends java.awt.Canvas {
		public Canvas() {
			setBackground(Color.black);
			setForeground(Color.green);
		}

		public void paint(Graphics g) {
			Dimension s=getSize();
			for(int i=0;i<buffer.length;i++) {
				g.drawLine(
					(i*s.width)/buffer.length,
					ordonnee(buffer[i],s.height),
					((i+1)*s.width)/buffer.length,
					ordonnee(buffer[(i+1)%buffer.length],s.height)
				);
			}
			synchronized(this) {
				notify();
			}
		}

		public Dimension getPreferredSize() {
			return getParent().getSize();
		}
	};

	public Oscilloscope() {
		setSize(128,64);
		setLayout(new BorderLayout());
		canvas=new Canvas();
		add(canvas);
		init();
	}

	public void run() {
		ComplexInput in=(ComplexInput)connector.getInputStream(0);

		stopped=false;
		while(!stopped) {
			try {
				for(int i=0;i<buffer.length;i++) buffer[i]=in.readComplex();
				synchronized(canvas) {
					canvas.repaint();
					try {
						canvas.wait();
					} catch (InterruptedException ex) {
						throw new java.io.InterruptedIOException();
					}
				}
			} catch (InterruptedIOException e) {
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void stop() {
		stopped=true;
	}

	int ordonnee(Complex c, int height) {
		double d=imaginaire?c.element1:c.element0;
		if(d>maximum) d=1.0;
		else if(d<minimum) d=0.0;
		else d=(d-minimum)/(maximum-minimum);
		return (int)((1.-d)*(double)height);
	}

	public void init() {
		buffer=new Complex[largeur];
		for(int i=0;i<buffer.length;i++) buffer[i]=Complex.valueOf(0.,0.);
	}

	public void open() {}

	public void close() {}

	public void connect(Connectable dest, OutputStream os, int nout) {
		connector.connect(dest,os,nout);
	}

	public void disconnect(Connectable dest) {
		connector.disconnect(dest);
	}

	public void connection(ConnectionEvent e) {
		connector.connection(e);
	}

	public void setLargeur(int largeur) {
		this.largeur=largeur;
		doLayout();
		init();
	}

	public int getLargeur() {
		return(largeur);
	}

	public void setMinimum(double minimum) {
		this.minimum=minimum;
		canvas.repaint();
	}

	public double getMinimum() {
		return(minimum);
	}

	public void setMaximum(double maximum) {
		this.maximum=maximum;
		canvas.repaint();
	}

	public double getMaximum() {
		return(maximum);
	}

	public void setImaginaire(boolean imaginaire) {
		this.imaginaire=imaginaire;
		canvas.repaint();
	}

	public boolean getImaginaire() {
		return(imaginaire);
	}

	private void writeObject(ObjectOutputStream s) throws IOException {
		s.defaultWriteObject();
	}

	private void readObject(ObjectInputStream s) throws ClassNotFoundException, IOException {
		s.defaultReadObject();
		init();
	}
}
