package spoke.space;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import spoke.math.numeric.*;

public class Viewer extends Applet implements Observer {
	Modele modele;
	int dimension;
	Matrix repere;
	Matrix decale;
	int x;
	int y;
	boolean shift=false;
	Image image[]=new Image[1];
	Graphics graphics[]=new Graphics[1];
	Dimension size;
	int width;

	MouseMotionAdapter mmadapter=new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {
			Dimension size=getSize();
			double tx=(x-e.getX())*(2.*Math.PI)/size.width;
			double ty=(e.getY()-y)*(2.*Math.PI)/size.height;
			if(shift && dimension>3) {
//				repere=(Matrix)Matrix.rotation(dimension,0,3,tx).multiply(repere);
//				repere=(Matrix)Matrix.rotation(dimension,1,3,ty).multiply(repere);
				repere=(Matrix)Matrix.rotation(dimension,2,3,ty).multiply(repere);
			} else {
				repere=(Matrix)Matrix.rotation(dimension,0,tx).multiply(repere);
				repere=(Matrix)Matrix.rotation(dimension,1,ty).multiply(repere);
			}
			repaint();
			x=e.getX();
			y=e.getY();
		}

		public void mouseMoved(MouseEvent e) {
			x=e.getX();
			y=e.getY();
		}
	};

	KeyAdapter kadapter=new KeyAdapter() {
		public void keyPressed(KeyEvent e) {
			if(e.getKeyCode()==KeyEvent.VK_SHIFT) shift=true;
		}

		public void keyReleased(KeyEvent e) {
			if(e.getKeyCode()==KeyEvent.VK_SHIFT) shift=false;
		}
	};

	public Viewer() {
		addMouseMotionListener(mmadapter);
		addKeyListener(kadapter);
		setBackground(Color.black);
	}

	public void setModele(Modele modele) {
		this.modele=modele;
		dimension=modele.getDimension();
		modele.addObserver(this);
		repere=Matrix.identity(dimension);
		decale=repere;
	}

	public void init() {
		try {
			setModele((Modele)Class.forName(getParameter("modele")).newInstance());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void start() {
		new Thread(modele).start();
	}

	public void update(Observable o, Object arg) {
		repaint();
		synchronized(this) {
			try {
				wait();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public void paint(Graphics g) {
		decale=repere;
		myUpdate(graphics[0]);
		g.drawImage(image[0],0,0,this);
		synchronized(this) {
			notify();
		}
	}

	void myPaint(Graphics g) {
		double data[][][]=modele.getData();

		for(int i=0;i<data.length;i++) {
			double p[][]=data[i];
			double q[][]=new double[p.length][];
			for(int j=0;j<p.length;j++) {
				q[j]=((SpokeVector)decale.multiply(new SpokeVector(p[j]))).element;
			}
			int x[][]=projection(q);
			g.setColor(Color.black);
			g.drawPolyline(x[0],x[1],q.length);
		}
	}

	int[][] projection(double p[][]) {
		int x[][]=new int[2][p.length];

		for(int i=0;i<p.length;i++) {
			double q[]=p[i];
			double z=(3.-q[2])/2.;
			x[0][i]=(int)(((q[0]/z)*.5+.5)*width);
			x[1][i]=(int)((.5-(q[1]/z)*.5)*size.height);
		}
		return x;
	}

	public void update(Graphics g) {
		paint(g);
	}

	void myUpdate(Graphics g) {
		g.setColor(getBackground());
		g.fillRect(0, 0, width, size.height);
		g.setColor(getForeground());
		myPaint(g);
	}

	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x,y,width,height);
		size=getSize();
		setImage();
	}

	void setImage() {
		image=new Image[1];
		graphics=new Graphics[1];
		width=size.width;
		image[0]=createImage(width,size.height);
		graphics[0]=image[0].getGraphics();
	}

	public Dimension getPreferredSize() {
		return new Dimension(200,200);
	}
}
